@@ -77,7 +77,7 @@ class DiskPartitioner(object):
         #                    need to log it again here.
         utils.execute('parted', '-a', self._alignment, '-s', self._device,
                       '--', 'unit', 'MiB', *args, check_exit_code=[0],
-                      run_as_root=True)
+                      use_standard_locale=True, run_as_root=True)
 
     def add_partition(self, size, part_type='primary', fs_type='',
                       bootable=False):
@@ -175,3 +175,8 @@ class DiskPartitioner(object):
                       'exited with "%(fuser_err)s". Time out waiting for '
                       'completion.')
                     % {'device': self._device, 'fuser_err': fuser_err[0]})
+
+
+def list_opts():
+    """Entry point for oslo-config-generator."""
+    return [('disk_partitioner', opts)]
