@@ -120,9 +120,17 @@ def get_disk_identifier(dev):
     return disk_identifier[0]
 
 
+def is_iscsi_device(dev, node_uuid):
+    """check whether the device path belongs to an iscsi device. """
+
+    iscsi_id = "iqn.2008-10.org.openstack:%s" % node_uuid
+    return iscsi_id in dev
+
+
 def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
                     configdrive_mb, node_uuid, commit=True,
-                    boot_option="netboot", boot_mode="bios"):
+                    boot_option="netboot", boot_mode="bios",
+                    disk_label=None):
     """Partition the disk device.
 
     Create partitions for root, swap, ephemeral and configdrive on a
@@ -140,6 +148,9 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     :param boot_option: Can be "local" or "netboot". "netboot" by default.
     :param boot_mode: Can be "bios" or "uefi". "bios" by default.
     :param node_uuid: Node's uuid. Used for logging.
+    :param disk_label: The disk label to be used when creating the
+        partition table. Valid values are: "msdos", "gpt" or None; If None
+        Ironic will figure it out according to the boot_mode parameter.
     :returns: A dictionary containing the partition type as Key and partition
         path as Value for the partitions created by this method.
 
@@ -147,19 +158,30 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     LOG.debug("Starting to partition the disk device: %(dev)s "
               "for node %(node)s",
               {'dev': dev, 'node': node_uuid})
-    part_template = dev + '-part%d'
+    # the actual device names in the baremetal are like /dev/sda, /dev/sdb etc.
+    # While for the iSCSI device, the naming convention has a format which has
+    # iqn also embedded in it.
+    # When this function is called by ironic-conductor, the iSCSI device name
+    # should be appended by "part%d". While on the baremetal, it should name
+    # the device partitions as /dev/sda1 and not /dev/sda-part1.
+    if is_iscsi_device(dev, node_uuid):
+        part_template = dev + '-part%d'
+    else:
+        part_template = dev + '%d'
     part_dict = {}
 
+    if disk_label is None:
+        disk_label = 'gpt' if boot_mode == 'uefi' else 'msdos'
+
+    dp = disk_partitioner.DiskPartitioner(dev, disk_label=disk_label)
+
     # For uefi localboot, switch partition table to gpt and create the efi
     # system partition as the first partition.
     if boot_mode == "uefi" and boot_option == "local":
-        dp = disk_partitioner.DiskPartitioner(dev, disk_label="gpt")
         part_num = dp.add_partition(CONF.disk_utils.efi_system_partition_size,
                                     fs_type='fat32',
                                     bootable=True)
         part_dict['efi system partition'] = part_template % part_num
-    else:
-        dp = disk_partitioner.DiskPartitioner(dev)
 
     if ephemeral_mb:
         LOG.debug("Add ephemeral partition (%(size)d MB) to device: %(dev)s "
@@ -284,49 +306,20 @@ def get_dev_block_size(dev):
 def destroy_disk_metadata(dev, node_uuid):
     """Destroy metadata structures on node's disk.
 
-       Ensure that node's disk appears to be blank without zeroing the entire
-       drive. To do this we will zero the first 18KiB to clear MBR / GPT data
-       and the last 18KiB to clear GPT and other metadata like LVM, veritas,
-       MDADM, DMRAID, etc.
+    Ensure that node's disk magic strings are wiped without zeroing the
+    entire drive. To do this we use the wipefs tool from util-linux.
+
+    :param dev: Path for the device to work on.
+    :param node_uuid: Node's uuid. Used for logging.
     """
     # NOTE(NobodyCam): This is needed to work around bug:
     # https://bugs.launchpad.net/ironic/+bug/1317647
     LOG.debug("Start destroy disk metadata for node %(node)s.",
               {'node': node_uuid})
-    try:
-        utils.dd('/dev/zero', dev, 'bs=512', 'count=36')
-    except processutils.ProcessExecutionError as err:
-        with excutils.save_and_reraise_exception():
-            LOG.error(_LE("Failed to erase beginning of disk for node "
-                          "%(node)s. Command: %(command)s. Error: %(error)s."),
-                      {'node': node_uuid,
-                       'command': err.cmd,
-                       'error': err.stderr})
-
-    # now wipe the end of the disk.
-    # get end of disk seek value
-    try:
-        block_sz = get_dev_block_size(dev)
-    except processutils.ProcessExecutionError as err:
-        with excutils.save_and_reraise_exception():
-            LOG.error(_LE("Failed to get disk block count for node %(node)s. "
-                          "Command: %(command)s. Error: %(error)s."),
-                      {'node': node_uuid,
-                       'command': err.cmd,
-                       'error': err.stderr})
-    else:
-        seek_value = block_sz - 36
-        try:
-            utils.dd('/dev/zero', dev, 'bs=512', 'count=36',
-                     'seek=%d' % seek_value)
-        except processutils.ProcessExecutionError as err:
-            with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to erase the end of the disk on node "
-                              "%(node)s. Command: %(command)s. "
-                              "Error: %(error)s."),
-                          {'node': node_uuid,
-                           'command': err.cmd,
-                           'error': err.stderr})
+    utils.execute('wipefs', '--all', dev,
+                  run_as_root=True,
+                  check_exit_code=[0],
+                  use_standard_locale=True)
     LOG.info(_LI("Disk metadata on %(dev)s successfully destroyed for node "
                  "%(node)s"), {'dev': dev, 'node': node_uuid})
 
@@ -394,7 +387,7 @@ def _get_configdrive(configdrive, node_uuid, tempdir=None):
 def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
                  image_path, node_uuid, preserve_ephemeral=False,
                  configdrive=None, boot_option="netboot", boot_mode="bios",
-                 tempdir=None):
+                 tempdir=None, disk_label=None):
     """Create partitions and copy an image to the root partition.
 
     :param dev: Path for the device to work on.
@@ -414,6 +407,9 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
     :param boot_option: Can be "local" or "netboot". "netboot" by default.
     :param boot_mode: Can be "bios" or "uefi". "bios" by default.
     :param tempdir: A temporary directory
+    :param disk_label: The disk label to be used when creating the
+        partition table. Valid values are: "msdos", "gpt" or None; If None
+        Ironic will figure it out according to the boot_mode parameter.
     :returns: a dictionary containing the following keys:
         'root uuid': UUID of root partition
         'efi system partition uuid': UUID of the uefi system partition
@@ -441,7 +437,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
                                     configdrive_mb, node_uuid,
                                     commit=commit,
                                     boot_option=boot_option,
-                                    boot_mode=boot_mode)
+                                    boot_mode=boot_mode,
+                                    disk_label=disk_label)
         LOG.info(_LI("Successfully completed the disk device"
                      " %(dev)s partitioning for node %(node)s"),
                  {'dev': dev, "node": node_uuid})
@@ -516,3 +513,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
             LOG.error(_LE("Failed to detect %s"), part)
 
     return uuids_to_return
+
+
+def list_opts():
+    """Entry point for oslo-config-generator."""
+    return [('disk_utils', opts)]
