@@ -24,6 +24,7 @@ import tempfile
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_service import loopingcall
 from oslo_utils import imageutils
 from oslotest import base as test_base
 import requests
@@ -115,7 +116,8 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
                                              self.configdrive_mb,
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
-                                             boot_mode="bios")
+                                             boot_mode="bios",
+                                             disk_label=None)
 
     def test_no_swap_partition(self):
         self.mock_ibd.side_effect = iter([True, False])
@@ -132,7 +134,8 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
                                              self.configdrive_mb,
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
-                                             boot_mode="bios")
+                                             boot_mode="bios",
+                                             disk_label=None)
 
     def test_no_ephemeral_partition(self):
         ephemeral_part = '/dev/fake-part1'
@@ -158,7 +161,8 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
                                              self.configdrive_mb,
                                              self.node_uuid, commit=True,
                                              boot_option="netboot",
-                                             boot_mode="bios")
+                                             boot_mode="bios",
+                                             disk_label=None)
 
     @mock.patch.object(utils, 'unlink_without_raise')
     @mock.patch.object(disk_utils, '_get_configdrive')
@@ -190,9 +194,40 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
                                              configdrive_mb, self.node_uuid,
                                              commit=True,
                                              boot_option="netboot",
-                                             boot_mode="bios")
+                                             boot_mode="bios",
+                                             disk_label=None)
         mock_unlink.assert_called_once_with('fake-path')
 
+    @mock.patch.object(utils, 'mkfs', lambda *_: None)
+    @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
+    @mock.patch.object(disk_utils, 'populate_image', lambda *_: None)
+    def test_gpt_disk_label(self):
+        ephemeral_part = '/dev/fake-part1'
+        swap_part = '/dev/fake-part2'
+        root_part = '/dev/fake-part3'
+        ephemeral_mb = 256
+        ephemeral_format = 'exttest'
+
+        self.mock_mp.return_value = {'ephemeral': ephemeral_part,
+                                     'swap': swap_part,
+                                     'root': root_part}
+        self.mock_ibd.return_value = True
+        calls = [mock.call(root_part),
+                 mock.call(swap_part),
+                 mock.call(ephemeral_part)]
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, ephemeral_mb, ephemeral_format,
+                                self.image_path, self.node_uuid,
+                                disk_label='gpt')
+        self.assertEqual(self.mock_ibd.call_args_list, calls)
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=True,
+                                             boot_option="netboot",
+                                             boot_mode="bios",
+                                             disk_label='gpt')
+
 
 @mock.patch.object(utils, 'execute')
 class MakePartitionsTestCase(test_base.BaseTestCase):
@@ -206,26 +241,27 @@ class MakePartitionsTestCase(test_base.BaseTestCase):
         self.configdrive_mb = 0
         self.node_uuid = "12345678-1234-1234-1234-1234567890abcxyz"
 
-    def _get_parted_cmd(self, dev):
+    def _get_parted_cmd(self, dev, label=None):
+        if label is None:
+            label = 'msdos'
+
         return ['parted', '-a', 'optimal', '-s', dev,
-                '--', 'unit', 'MiB', 'mklabel', 'msdos']
+                '--', 'unit', 'MiB', 'mklabel', label]
 
-    def _get_parted_cmd_uefi(self, dev):
-        return ['parted', '-a', 'optimal', '-s',
-                dev, '--', 'unit', 'MiB', 'mklabel', 'gpt']
-
-    def _test_make_partitions(self, mock_exc, boot_option):
+    def _test_make_partitions(self, mock_exc, boot_option, disk_label=None):
         mock_exc.return_value = (None, None)
         disk_utils.make_partitions(self.dev, self.root_mb, self.swap_mb,
                                    self.ephemeral_mb, self.configdrive_mb,
-                                   self.node_uuid, boot_option=boot_option)
+                                   self.node_uuid, boot_option=boot_option,
+                                   disk_label=disk_label)
 
         expected_mkpart = ['mkpart', 'primary', 'linux-swap', '1', '513',
                            'mkpart', 'primary', '', '513', '1537']
         if boot_option == "local":
             expected_mkpart.extend(['set', '2', 'boot', 'on'])
         self.dev = 'fake-dev'
-        parted_cmd = self._get_parted_cmd(self.dev) + expected_mkpart
+        parted_cmd = (self._get_parted_cmd(self.dev, disk_label) +
+                      expected_mkpart)
         parted_call = mock.call(*parted_cmd, use_standard_locale=True,
                                 run_as_root=True, check_exit_code=[0])
         fuser_cmd = ['fuser', 'fake-dev']
@@ -239,6 +275,10 @@ class MakePartitionsTestCase(test_base.BaseTestCase):
     def test_make_partitions_local_boot(self, mock_exc):
         self._test_make_partitions(mock_exc, boot_option="local")
 
+    def test_make_partitions_disk_label_gpt(self, mock_exc):
+        self._test_make_partitions(mock_exc, boot_option="netboot",
+                                   disk_label="gpt")
+
     def test_make_partitions_with_ephemeral(self, mock_exc):
         self.ephemeral_mb = 2048
         expected_mkpart = ['mkpart', 'primary', '', '1', '2049',
@@ -255,15 +295,40 @@ class MakePartitionsTestCase(test_base.BaseTestCase):
                                 run_as_root=True, check_exit_code=[0])
         mock_exc.assert_has_calls([parted_call])
 
+    def test_make_partitions_with_iscsi_device(self, mock_exc):
+        self.ephemeral_mb = 2048
+        expected_mkpart = ['mkpart', 'primary', '', '1', '2049',
+                           'mkpart', 'primary', 'linux-swap', '2049', '2561',
+                           'mkpart', 'primary', '', '2561', '3585']
+        self.dev = '/dev/iqn.2008-10.org.openstack:%s.fake-9' % self.node_uuid
+        ep = '/dev/iqn.2008-10.org.openstack:%s.fake-9-part1' % self.node_uuid
+        swap = ('/dev/iqn.2008-10.org.openstack:%s.fake-9-part2'
+                % self.node_uuid)
+        root = ('/dev/iqn.2008-10.org.openstack:%s.fake-9-part3'
+                % self.node_uuid)
+        expected_result = {'ephemeral': ep,
+                           'swap': swap,
+                           'root': root}
+        cmd = self._get_parted_cmd(self.dev) + expected_mkpart
+        mock_exc.return_value = (None, None)
+        result = disk_utils.make_partitions(
+            self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
+            self.configdrive_mb, self.node_uuid)
+
+        parted_call = mock.call(*cmd, use_standard_locale=True,
+                                run_as_root=True, check_exit_code=[0])
+        mock_exc.assert_has_calls([parted_call])
+        self.assertEqual(expected_result, result)
+
     def test_make_partitions_with_local_device(self, mock_exc):
         self.ephemeral_mb = 2048
         expected_mkpart = ['mkpart', 'primary', '', '1', '2049',
                            'mkpart', 'primary', 'linux-swap', '2049', '2561',
                            'mkpart', 'primary', '', '2561', '3585']
         self.dev = 'fake-dev'
-        expected_result = {'ephemeral': 'fake-dev-part1',
-                           'swap': 'fake-dev-part2',
-                           'root': 'fake-dev-part3'}
+        expected_result = {'ephemeral': 'fake-dev1',
+                           'swap': 'fake-dev2',
+                           'root': 'fake-dev3'}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
         mock_exc.return_value = (None, None)
         result = disk_utils.make_partitions(
@@ -276,7 +341,6 @@ class MakePartitionsTestCase(test_base.BaseTestCase):
         self.assertEqual(expected_result, result)
 
 
-@mock.patch.object(disk_utils, 'get_dev_block_size')
 @mock.patch.object(utils, 'execute')
 class DestroyMetaDataTestCase(test_base.BaseTestCase):
 
@@ -285,39 +349,19 @@ class DestroyMetaDataTestCase(test_base.BaseTestCase):
         self.dev = 'fake-dev'
         self.node_uuid = "12345678-1234-1234-1234-1234567890abcxyz"
 
-    def test_destroy_disk_metadata(self, mock_exec, mock_gz):
-        mock_gz.return_value = 64
-        expected_calls = [mock.call('dd', 'if=/dev/zero', 'of=fake-dev',
-                                    'bs=512', 'count=36', run_as_root=True,
-                                    check_exit_code=[0],
-                                    use_standard_locale=True),
-                          mock.call('dd', 'if=/dev/zero', 'of=fake-dev',
-                                    'bs=512', 'count=36', 'seek=28',
+    def test_destroy_disk_metadata(self, mock_exec):
+        expected_calls = [mock.call('wipefs', '--all', 'fake-dev',
                                     run_as_root=True,
                                     check_exit_code=[0],
                                     use_standard_locale=True)]
         disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
         mock_exec.assert_has_calls(expected_calls)
-        self.assertTrue(mock_gz.called)
 
-    def test_destroy_disk_metadata_get_dev_size_fail(self, mock_exec, mock_gz):
-        mock_gz.side_effect = processutils.ProcessExecutionError
-
-        expected_call = [mock.call('dd', 'if=/dev/zero', 'of=fake-dev',
-                                   'bs=512', 'count=36', run_as_root=True,
-                                   check_exit_code=[0],
-                                   use_standard_locale=True)]
-        self.assertRaises(processutils.ProcessExecutionError,
-                          disk_utils.destroy_disk_metadata,
-                          self.dev,
-                          self.node_uuid)
-        mock_exec.assert_has_calls(expected_call)
-
-    def test_destroy_disk_metadata_dd_fail(self, mock_exec, mock_gz):
+    def test_destroy_disk_metadata_wipefs_fail(self, mock_exec):
         mock_exec.side_effect = processutils.ProcessExecutionError
 
-        expected_call = [mock.call('dd', 'if=/dev/zero', 'of=fake-dev',
-                                   'bs=512', 'count=36', run_as_root=True,
+        expected_call = [mock.call('wipefs', '--all', 'fake-dev',
+                                   run_as_root=True,
                                    check_exit_code=[0],
                                    use_standard_locale=True)]
         self.assertRaises(processutils.ProcessExecutionError,
@@ -325,7 +369,6 @@ class DestroyMetaDataTestCase(test_base.BaseTestCase):
                           self.dev,
                           self.node_uuid)
         mock_exec.assert_has_calls(expected_call)
-        self.assertFalse(mock_gz.called)
 
 
 @mock.patch.object(utils, 'execute')
@@ -367,6 +410,13 @@ class PopulateImageTestCase(test_base.BaseTestCase):
         self.assertFalse(mock_dd.called)
 
 
+def _looping_call_done(*args, **kwargs):
+    raise loopingcall.LoopingCallDone()
+
+
+@mock.patch.object(disk_partitioner.DiskPartitioner,
+                   '_wait_for_disk_to_become_available',
+                   _looping_call_done)
 @mock.patch.object(disk_utils, 'is_block_device', lambda d: True)
 @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
 @mock.patch.object(disk_utils, 'dd', lambda *_: None)
